>From b58b921d4413c23947b9a5c2c311d4828e89bfd5 Mon Sep 17 00:00:00 2001
From: Andres Freund <andres@anarazel.de>
Date: Thu, 27 Feb 2014 15:18:44 +0100
Subject: [PATCH] Improve consistency in replication slot data types.

Replication slots sometimes referred to the slot name using the type
text, sometimes the type name, standarize on the latter. Also adjust
pg_replication_slot.datoid to return NULL for slots that are not
database specific, database already was NULL in such cases.

Adjust the docs to match, and fix the oversight that
pg_drop_replication_slot() now returns void instead of the slot name
and that pg_create_physical_replication_slot() doesn't have `plugin'
parameter. Those oversights were noticed by Vik Fearing.

Requires a catversion bump because of the return type changes.
---
 doc/src/sgml/func.sgml              |  8 ++++----
 doc/src/sgml/high-availability.sgml |  2 +-
 src/backend/replication/slotfuncs.c | 14 +++++++++-----
 src/include/catalog/pg_proc.h       |  4 ++--
 4 files changed, 16 insertions(+), 12 deletions(-)

diff --git a/doc/src/sgml/func.sgml b/doc/src/sgml/func.sgml
index ff50328..d344b3a 100644
--- a/doc/src/sgml/func.sgml
+++ b/doc/src/sgml/func.sgml
@@ -16317,10 +16317,10 @@ postgres=# SELECT * FROM pg_xlogfile_name_offset(pg_stop_backup());
         <indexterm>
          <primary>pg_create_physical_replication_slot</primary>
         </indexterm>
-        <literal><function>pg_create_physical_replication_slot(<parameter>slotname</parameter> <type>text</type>, <parameter>plugin</parameter> <type>text</type>)</function></literal>
+        <literal><function>pg_create_physical_replication_slot(<parameter>slotname</parameter> <type>name</type>)</function></literal>
        </entry>
        <entry>
-        (<parameter>slotname</parameter> <type>text</type>, <parameter>xlog_position</parameter> <type>pg_lsn</type>)
+        (<parameter>slotname</parameter> <type>name</type>, <parameter>xlog_position</parameter> <type>pg_lsn</type>)
        </entry>
        <entry>
         Creates a new physical replication slot named
@@ -16335,10 +16335,10 @@ postgres=# SELECT * FROM pg_xlogfile_name_offset(pg_stop_backup());
         <indexterm>
          <primary>pg_drop_replication_slot</primary>
         </indexterm>
-        <literal><function>pg_drop_replication_slot(<parameter>slotname</parameter> <type>text</type>)</function></literal>
+        <literal><function>pg_drop_replication_slot(<parameter>slotname</parameter> <type>name</type>)</function></literal>
        </entry>
        <entry>
-        (<parameter>slotname</parameter> <type>text</type>)
+        <type>void</type>
        </entry>
        <entry>
         Drops the physical or logical replication slot
diff --git a/doc/src/sgml/high-availability.sgml b/doc/src/sgml/high-availability.sgml
index da17455..ecb51c4 100644
--- a/doc/src/sgml/high-availability.sgml
+++ b/doc/src/sgml/high-availability.sgml
@@ -935,7 +935,7 @@ postgres=# SELECT * FROM pg_create_physical_replication_slot('node_a_slot');
 postgres=# SELECT * FROM pg_replication_slots;
   slot_name  | slot_type | datoid | database | active | xmin | restart_lsn
 -------------+-----------+--------+----------+--------+------+-------------
- node_a_slot | physical  |      0 |          | f      |      |
+ node_a_slot | physical  |        |          | f      |      |
 (1 row)
 </programlisting>
      To configure the standby to use this slot, <varname>primary_slotname</>
diff --git a/src/backend/replication/slotfuncs.c b/src/backend/replication/slotfuncs.c
index 761c0f5..5acd2ba 100644
--- a/src/backend/replication/slotfuncs.c
+++ b/src/backend/replication/slotfuncs.c
@@ -56,7 +56,7 @@ pg_create_physical_replication_slot(PG_FUNCTION_ARGS)
 	/* acquire replication slot, this will check for conflicting names*/
 	ReplicationSlotCreate(NameStr(*name), false);
 
-	values[0] = CStringGetTextDatum(NameStr(MyReplicationSlot->data.name));
+	values[0] = NameGetDatum(&MyReplicationSlot->data.name);
 
 	nulls[0] = false;
 	nulls[1] = true;
@@ -141,7 +141,8 @@ pg_get_replication_slots(PG_FUNCTION_ARGS)
 		XLogRecPtr	restart_lsn;
 		bool		active;
 		Oid			database;
-		const char *slot_name;
+		NameData	slot_name;
+
 		int			i;
 
 		SpinLockAcquire(&slot->mutex);
@@ -155,7 +156,7 @@ pg_get_replication_slots(PG_FUNCTION_ARGS)
 			xmin = slot->data.xmin;
 			database = slot->data.database;
 			restart_lsn = slot->data.restart_lsn;
-			slot_name = pstrdup(NameStr(slot->data.name));
+			namecpy(&slot_name, &slot->data.name);
 
 			active = slot->active;
 		}
@@ -164,12 +165,15 @@ pg_get_replication_slots(PG_FUNCTION_ARGS)
 		memset(nulls, 0, sizeof(nulls));
 
 		i = 0;
-		values[i++] = CStringGetTextDatum(slot_name);
+		values[i++] = NameGetDatum(&slot_name);
 		if (database == InvalidOid)
 			values[i++] = CStringGetTextDatum("physical");
 		else
 			values[i++] = CStringGetTextDatum("logical");
-		values[i++] = database;
+		if (database == InvalidOid)
+			nulls[i++] = true;
+		else
+			values[i++] = database;
 		values[i++] = BoolGetDatum(active);
 		if (xmin != InvalidTransactionId)
 			values[i++] = TransactionIdGetDatum(xmin);
diff --git a/src/include/catalog/pg_proc.h b/src/include/catalog/pg_proc.h
index e33670c..7a11721 100644
--- a/src/include/catalog/pg_proc.h
+++ b/src/include/catalog/pg_proc.h
@@ -4800,11 +4800,11 @@ DATA(insert OID = 3473 (  spg_range_quad_leaf_consistent	PGNSP PGUID 12 1 0 0 0
 DESCR("SP-GiST support for quad tree over range");
 
 /* replication slots */
-DATA(insert OID = 3779 (  pg_create_physical_replication_slot PGNSP PGUID 12 1 0 0 0 f f f f f f v 1 0 2249 "19" "{19,25,3220}" "{i,o,o}" "{slotname,slotname,xlog_position}" _null_ pg_create_physical_replication_slot _null_ _null_ _null_ ));
+DATA(insert OID = 3779 (  pg_create_physical_replication_slot PGNSP PGUID 12 1 0 0 0 f f f f f f v 1 0 2249 "19" "{19,19,3220}" "{i,o,o}" "{slotname,slotname,xlog_position}" _null_ pg_create_physical_replication_slot _null_ _null_ _null_ ));
 DESCR("create a physical replication slot");
 DATA(insert OID = 3780 (  pg_drop_replication_slot PGNSP PGUID 12 1 0 0 0 f f f f f f v 1 0 2278 "19" _null_ _null_ _null_ _null_ pg_drop_replication_slot _null_ _null_ _null_ ));
 DESCR("drop a replication slot");
-DATA(insert OID = 3781 (  pg_get_replication_slots	PGNSP PGUID 12 1 10 0 0 f f f f f t s 0 0 2249 "" "{25,25,26,16,28,3220}" "{o,o,o,o,o,o}" "{slot_name,slot_type,datoid,active,xmin,restart_lsn}" _null_ pg_get_replication_slots _null_ _null_ _null_ ));
+DATA(insert OID = 3781 (  pg_get_replication_slots	PGNSP PGUID 12 1 10 0 0 f f f f f t s 0 0 2249 "" "{19,25,26,16,28,3220}" "{o,o,o,o,o,o}" "{slot_name,slot_type,datoid,active,xmin,restart_lsn}" _null_ pg_get_replication_slots _null_ _null_ _null_ ));
 DESCR("information about replication slots currently in use");
 
 /* event triggers */
-- 
1.8.3.251.g1462b67

